
def extract_from_doc(pdf_path, page_num, pdf_type):
    """
    Extracts text from a PDF file.
    """
    raise NotImplementedError


def extract_from_image(image_path):
    """
    Extracts text from an image file.
    """
    raise NotImplementedError


def extract_from_webpage(url):
    """
    Extracts text from a webpage.
    """
    raise NotImplementedError
